#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import cv2, time
import bottle
import os

app = bottle.Bottle()

if len(sys.argv) > 1:
    raspi_ip = os.environ['RASPI_IP']
    url = "http://"+raspi_ip+":40080/?action=stream"
else:
    url = "input.mp4"

print("play "+url)


cascPath = './lbpcascades/lbpcascade_frontalface.xml'
#gpu classifier = cv2.cuda.CascadeClassifier_create(cascPath)
classifier = cv2.CascadeClassifier(cascPath)

def gen():
    cap=cv2.VideoCapture(url)

    width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    fps = cap.get(cv2.CAP_PROP_FPS)

    width_mosaic = int(width / 48)
    height_mosaic = int(height / 48)

    disp_scale = 1/4;
    #gpu gpu_frame  = cv2.cuda_GpuMat()
    #gpu gpu_frame_mosaic  = cv2.cuda_GpuMat()

    fmt = cv2.VideoWriter_fourcc('m', 'p', '4', 'v')
    writer = cv2.VideoWriter('./output.mp4', fmt, fps, (int(width*disp_scale), int(height*disp_scale)))

    first_time = time.perf_counter()
    last_time = time.perf_counter()
    disp_last_time = time.perf_counter()
    disp_fps_str = ""
    frame_count = 0

    while True:
        current_time = time.perf_counter()
        fps_measurement = round(1 / (current_time - last_time), 1)
        last_time = current_time
        frame_count +=1
        print("Frame: "+ str(frame_count)+" "+str(fps_measurement)+" fps")

        ret, frame = cap.read()
    
        if not ret:
            print("done")
            cap.release()
            writer.release()
            cv2.destroyAllWindows()
            break
    
        gray_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
#gpu        gpu_gray_frame  = cv2.cuda_GpuMat(gray_frame)
#gpu        gpu_result = classifier.detectMultiScale(gpu_gray_frame)
#gpu        result = gpu_result.download()
        result = classifier.detectMultiScale(gray_frame)
    
    
#gpu        gpu_frame.upload(frame)
#gpu        gpu_frame_mosaic = cv2.cuda.resize(gpu_frame, (width_mosaic, height_mosaic))
#gpu        gpu_frame_mosaic = cv2.cuda.resize(gpu_frame_mosaic, (int(width*disp_scale), int(height*disp_scale)))
#gpu        disp_frame_mosaic = gpu_frame_mosaic.download();
        frame_mosaic = cv2.resize(frame, (width_mosaic, height_mosaic))
        disp_frame_mosaic = cv2.resize(frame_mosaic, (int(width*disp_scale), int(height*disp_scale)))
    
#gpu        if result is not None:
#gpu            for item in result[0]:
#gpu                (x, y, w, h) = item
#gpu                cv2.rectangle(disp_frame_mosaic, (int(x*disp_scale), int(y*disp_scale)), (int((x+w)*disp_scale), int((y+h)*disp_scale)), (0, 255, 0), 2)
        if len(result)>0: #  >0
            for rect in result:
                x,y,w,h=rect
                p1,p2=(int(x*disp_scale),int(y*disp_scale)),(int((x+w)*disp_scale),int((y+h)*disp_scale))
                cv2.rectangle(disp_frame_mosaic,p1,p2,color=(0,255,0),thickness=4)
                
        if (current_time - disp_last_time) > 0.2:
            disp_fps_str = str(fps_measurement)+"fps"+"/"+str(round(frame_count / (current_time - first_time), 1))
            disp_last_time = current_time

        cv2.putText(disp_frame_mosaic, disp_fps_str, (1,21), fontFace = cv2.FONT_HERSHEY_COMPLEX_SMALL, fontScale = 1, color = (255,255,255))
        cv2.putText(disp_frame_mosaic, disp_fps_str, (0,20), fontFace = cv2.FONT_HERSHEY_COMPLEX_SMALL, fontScale = 1, color = (0,255,0))
            
        writer.write(disp_frame_mosaic)
    
        flag, frame_jpeg = cv2.imencode('.jpg', disp_frame_mosaic)
        yield b'--frame\r\n' + b'Content-Type: image/jpeg\r\n\r\n' + bytearray(frame_jpeg) + b'\r\n\r\n'

#    cv2.imshow("Camera GPUTest", disp_frame_mosaic)
#    if cv2.waitKey(1) & 0xFF == ord('q'):
#        break
    
    
@app.route('/')
def main():
    return bottle.static_file('cpu.html', root='./www')

@app.route('/video_feed')
def video_feed():
    bottle.response.content_type = 'multipart/x-mixed-replace;boundary=frame'
    return gen()

app.run(host='0.0.0.0', port=9090, reloader=True, debug=True)
